/*
Copyright 2012 Daniel Morton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Daniel Morton ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Daniel Morton OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Daniel Morton.
*/
package ca.quadrilateral.blog.embeddedderbyintomcat.web;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

public class DerbyDBInitializingListener implements ServletContextListener {
    private static final Logger log = Logger.getLogger(DerbyDBInitializingListener.class.getName());
    private static final String TEST_TABLE_NAME = "product";

    private static final List<String> seedSQL = Arrays.asList(
            "CREATE TABLE product (" +
            "    id INTEGER NOT NULL PRIMARY KEY GENERATED ALWAYS AS IDENTITY, " +
            "    name VARCHAR(50), " +
            "    manufacturer VARCHAR(50), " +
            "    number_in_stock INTEGER, " +
            "    cost NUMERIC(6,2))",
            "INSERT INTO product (name, manufacturer, number_in_stock, cost) VALUES ('Gloves', 'Gloria''s Glove Emporium', 9, 44.99)",
            "INSERT INTO product (name, manufacturer, number_in_stock, cost) VALUES ('Boots', 'Baden Boots', 5, 139.99)",
            "INSERT INTO product (name, manufacturer, number_in_stock, cost) VALUES ('Hat', 'High Class Haberdashery', 80, 29.99)",
            "INSERT INTO product (name, manufacturer, number_in_stock, cost) VALUES ('Scarf', 'Scarf Stop', 0, 10.99)"
                );

    @Override
    public void contextInitialized(ServletContextEvent event) {
        log.info("Servlet Content Initialized");

        try {
            log.info("Loading Derby DB Driver...");
            Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
            initializeDatabase();
        } catch (ClassNotFoundException e) {
            log.log(Level.SEVERE, "Could not load Derby Embedded Driver!", e);
        } catch (SQLException sqle) {
            log.log(Level.SEVERE, "Fatal Database Error!", sqle);
        }
    }

    private void initializeDatabase() throws SQLException {
        Connection connection = null;
        try {
            log.info("Starting up Derby DB...");
            connection = DriverManager
                    .getConnection("jdbc:derby:databaseName;create=true");
            if (!schemaHasBeenInitialized(connection)) {
                initializeSchema(connection);
            }
        } catch (SQLException sqle) {
            log.log(Level.SEVERE, "Could not connect to Derby Embedded DB!", sqle);
            throw sqle;
        } finally {
            if (connection != null) {
                connection.close();
            }
        }
    }

    private boolean schemaHasBeenInitialized(Connection con) throws SQLException {
        final DatabaseMetaData metaData = con.getMetaData();
        ResultSet tablesResultSet = null;

        try {
            tablesResultSet =
                    metaData.getTables(
                            null, null, null,
                            new String[] { "TABLE" });

            while (tablesResultSet.next()) {
                final String tableName = tablesResultSet.getString("TABLE_NAME");
                if (tableName != null
                        && TEST_TABLE_NAME.equalsIgnoreCase(tableName)) {
                    log.info("Schema has already been initialized...");
                    return true;
                }
            }
        } finally {
            if (tablesResultSet != null) {
                tablesResultSet.close();
            }
        }
        log.info("Schema has not already been initialized...");
        return false;
    }

    private void initializeSchema(Connection con) {
        log.info("Initializing Database Schema...");
        Statement statement = null;
        try {
            statement = con.createStatement();
            for(String sql : seedSQL) {
                statement.execute(sql);
            }
        } catch (SQLException sqle) {
            log.severe("Error executing database initialization SQL: " + sqle.getMessage());
        } finally {
            if (statement != null) {
                try {
                    statement.close();
                } catch (Exception e) {
                    log.warning("Error closing statement resource: " + e.getMessage());
                }
            }
        }
    }

    @Override
    public void contextDestroyed(ServletContextEvent event) {
        log.info("Servlet Context Destroyed");
        try {
            log.info("Shutting down Derby DB...");
            DriverManager.getConnection("jdbc:derby:databaseName;shutdown=true");
        } catch (SQLException sqle) {
            if (sqle.getMessage().equals("Database 'databaseName' shutdown.")) {
                log.info("Derby DB Shutdown successfully!");
            } else {
                throw new RuntimeException(
                    "An error occurred shutting down the Derby instance!"
                    , sqle);
            }
        }
    }


}
